MODULE Automat;

(* Implementierung des Objektmoduls fr einen Getraenkeautomaten.
   Autor           : Moritz Schnizler, RWTH Aachen
   Umgebung        : SRC-Modula-3 rel. 3.6, Windows NT 4.0
   Erstellt        : 21.01.99  Letzte Aenderung: 21.01.99 *)

IMPORT Assertion;


CONST MaxGetraenke = 5; (* Maximale Anzahl Getraenke im Automaten *)


VAR getraenke   : [0 .. MaxGetraenke];
    mEingeworfen: BOOLEAN;


PROCEDURE MuenzeEinwerfen() RAISES {Assertion.Violated}=
(* Einwerfen einer Muenze in den Automaten. *)
BEGIN
  Assertion.Require(NOT MuenzeEingeworfen(), "MuenzeEinwerfen");
  mEingeworfen := TRUE;
  Assertion.Ensure(MuenzeEingeworfen(), "MuenzeEinwerfen");
END MuenzeEinwerfen;

PROCEDURE GeldZurueck() RAISES {Assertion.Violated}=
(* Gibt eine eventuell eingeworfene Muenze zurueck. *)
BEGIN
  Assertion.Require(MuenzeEingeworfen(), "GeldZurueck");
  mEingeworfen := FALSE;
  Assertion.Ensure(NOT MuenzeEingeworfen(), "GeldZurueck");
END GeldZurueck;

PROCEDURE Auffuellen() RAISES {Assertion.Violated}=
(* Fuelle ein Getraenk in den Automaten ein. *)
BEGIN
  Assertion.Require(NOT Voll(), "Auffuellen");
  getraenke := getraenke + 1;
  Assertion.Ensure(NOT Leer(), "Auffuellen");
END Auffuellen;

PROCEDURE Entnehmen() RAISES {Assertion.Violated}=
(* Entnehme bezahltes Getraenk aus dem Automaten. *)
BEGIN
  Assertion.Require(MuenzeEingeworfen() AND NOT Leer(), "Entnehmen");
  getraenke := getraenke - 1;
  mEingeworfen := FALSE;
  Assertion.Ensure(NOT MuenzeEingeworfen() AND NOT Voll(), "Entnehmen");
END Entnehmen;

PROCEDURE MuenzeEingeworfen(): BOOLEAN=
(* Prueft, ob Muenze eingeworfen wurde. *)
BEGIN
  RETURN mEingeworfen;
END MuenzeEingeworfen;

PROCEDURE Voll(): BOOLEAN=
(* Prueft, ob der Automat voll ist. *)
BEGIN
  RETURN (getraenke = MaxGetraenke);
END Voll;

PROCEDURE Leer(): BOOLEAN=
(* Prueft, ob der Automat leer ist. *)
BEGIN
  RETURN (getraenke = 0);
END Leer;

BEGIN
  (* Initialisiere den Automaten als leer. *)
  getraenke := 0;
  mEingeworfen := FALSE;
END Automat.